From 08765f4f61d63398307e3401313c45a60d65edee Mon Sep 17 00:00:00 2001
From: Yuan Fu <casouri@gmail.com>
Date: Sun, 5 Feb 2023 20:22:52 -0800
Subject: [PATCH] Demo

---
 src/treesit.c | 26 +++++++++++++++++++++++---
 1 file changed, 23 insertions(+), 3 deletions(-)

diff --git a/src/treesit.c b/src/treesit.c
index 8e772523cc7..f2c3e9845d1 100644
--- a/src/treesit.c
+++ b/src/treesit.c
@@ -1471,6 +1471,19 @@ DEFUN ("treesit-parser-language",
   return XTS_PARSER (parser)->language_symbol;
 }
 
+DEFUN ("treesit-parser-live-p",
+       Ftreesit_parser_live_p, Streesit_parser_live_p, 1, 1, 0,
+       doc: /* Check whether PARSER is not deleted and its buffer is live.  */)
+  (Lisp_Object parser)
+{
+  if (XTS_PARSER (parser)->deleted)
+    return Qnil;
+  else if (NILP (Fbuffer_live_p (XTS_PARSER (parser)->buffer)))
+    return Qnil;
+  else
+    return Qt;
+}
+
 /*** Parser API */
 
 DEFUN ("treesit-parser-root-node",
@@ -1904,7 +1917,8 @@ DEFUN ("treesit-node-check",
        Ftreesit_node_check, Streesit_node_check, 2, 2, 0,
        doc: /* Return non-nil if NODE has PROPERTY, nil otherwise.
 
-PROPERTY could be `named', `missing', `extra', `outdated', or `has-error'.
+PROPERTY could be `named', `missing', `extra', `outdated',
+`has-error', or `live'.
 
 Named nodes correspond to named rules in the language definition,
 whereas "anonymous" nodes correspond to string literals in the
@@ -1920,7 +1934,9 @@ DEFUN ("treesit-node-check",
 the node was created.
 
 A node "has error" if itself is a syntax error or contains any syntax
-errors.  */)
+errors.
+
+A node is "live" if its parser is live (i.e., not deleted).  */)
   (Lisp_Object node, Lisp_Object property)
 {
   if (NILP (node)) return Qnil;
@@ -1943,9 +1959,11 @@ DEFUN ("treesit-node-check",
     result = ts_node_is_extra (treesit_node);
   else if (EQ (property, Qhas_error))
     result = ts_node_has_error (treesit_node);
+  else if (EQ (property, Qlive))
+    result = Ftreesit_parser_live_p (XTS_NODE (node)->parser);
   else
     signal_error ("Expecting `named', `missing', `extra', "
-		  "`outdated', or `has-error', but got",
+                  "`outdated', `has-error', or `deleted', but got",
 		  property);
   return result ? Qt : Qnil;
 }
@@ -3444,6 +3462,7 @@ syms_of_treesit (void)
   DEFSYM (Qextra, "extra");
   DEFSYM (Qoutdated, "outdated");
   DEFSYM (Qhas_error, "has-error");
+  DEFSYM (Qlive, "live");
 
   DEFSYM (QCanchor, ":anchor");
   DEFSYM (QCequal, ":equal");
@@ -3577,6 +3596,7 @@ syms_of_treesit (void)
   defsubr (&Streesit_parser_list);
   defsubr (&Streesit_parser_buffer);
   defsubr (&Streesit_parser_language);
+  defsubr (&Streesit_parser_live_p);
 
   defsubr (&Streesit_parser_root_node);
   /* defsubr (&Streesit_parse_string); */
-- 
2.33.1

